@@ -1,5 +1,5 @@
 /*
- * Copyright (C) Pedram Pourang (aka Tsu Jan) 2014 <tsujan2000@gmail.com>
+ * Copyright (C) Pedram Pourang (aka Tsu Jan) 2014-2019 <tsujan2000@gmail.com>
  *
  * FeatherPad is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -23,7 +23,8 @@
 #include "pref.h"
 #include "ui_predDialog.h"
 
-#include <QDesktopWidget>
+#include <QScreen>
+#include <QWindow>
 #include <QWhatsThis>
 #include <QKeySequenceEdit>
 #include <QFileInfo>
@@ -31,6 +32,7 @@
 namespace FeatherPad {
 
 static QHash<QString, QString> OBJECT_NAMES;
+static QHash<QString, QString> DEFAULT_SHORTCUTS;
 
 Delegate::Delegate (QObject *parent)
     : QStyledItemDelegate (parent)
@@ -60,7 +62,7 @@ bool Delegate::eventFilter (QObject *object, QEvent *event)
     return QStyledItemDelegate::eventFilter (object, event);
 }
 /*************************/
-PrefDialog::PrefDialog (const QHash<QString, QString> &defaultShortcuts, QWidget *parent)
+PrefDialog::PrefDialog (QWidget *parent)
     : QDialog (parent), ui (new Ui::PrefDialog)
 {
     ui->setupUi (this);
@@ -85,7 +87,7 @@ PrefDialog::PrefDialog (const QHash<QString, QString> &defaultShortcuts, QWidget
     showWhiteSpace_ = config.getShowWhiteSpace();
     showEndings_ = config.getShowEndings();
     vLineDistance_ = config.getVLineDistance();
-    textTabSize_ = config.getTextTab().size();
+    textTabSize_ = config.getTextTabSize();
 
     /**************
      *** Window ***
@@ -100,15 +102,23 @@ PrefDialog::PrefDialog (const QHash<QString, QString> &defaultShortcuts, QWidget
         ui->mLabel->setEnabled (false);
         ui->sizeLable->setEnabled (false);
     }
-    QSize ag = QApplication::desktop()->availableGeometry().size();
+    QSize ag;
+    if (parent != nullptr)
+    {
+        if (QWindow *win = parent->windowHandle())
+        {
+            if (QScreen *sc = win->screen())
+                ag = sc->availableVirtualGeometry().size();
+        }
+    }
+    if (!ag.isValid()) ag = QSize(0, 0);
     ui->spinX->setMaximum (ag.width());
     ui->spinY->setMaximum (ag.height());
     ui->spinX->setValue (config.getStartSize().width());
     ui->spinY->setValue (config.getStartSize().height());
-    connect (ui->spinX, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),
-             this, &PrefDialog::prefStartSize);
-    connect (ui->spinY, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),
-             this, &PrefDialog::prefStartSize);
+    /* old-fashioned: connect (ui->spinX, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),... */
+    connect (ui->spinX, QOverload<int>::of(&QSpinBox::valueChanged), this, &PrefDialog::prefStartSize);
+    connect (ui->spinY, QOverload<int>::of(&QSpinBox::valueChanged), this, &PrefDialog::prefStartSize);
 
     ui->winPosBox->setChecked (config.getRemPos());
     connect (ui->winPosBox, &QCheckBox::stateChanged, this, &PrefDialog::prefPos);
@@ -188,8 +198,7 @@ PrefDialog::PrefDialog (const QHash<QString, QString> &defaultShortcuts, QWidget
     connect (ui->vLineBox, &QCheckBox::stateChanged, this, &PrefDialog::prefVLine);
     ui->vLineSpin->setEnabled (vLineDistance_ >= 10);
     ui->vLineSpin->setValue (qAbs (vLineDistance_));
-    connect (ui->vLineSpin, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),
-             this, &PrefDialog::prefVLineDistance);
+    connect (ui->vLineSpin, QOverload<int>::of(&QSpinBox::valueChanged), this, &PrefDialog::prefVLineDistance);
 
     ui->endingsBox->setChecked (config.getShowEndings());
     connect (ui->endingsBox, &QCheckBox::stateChanged, this, &PrefDialog::prefEndings);
@@ -208,8 +217,7 @@ PrefDialog::PrefDialog (const QHash<QString, QString> &defaultShortcuts, QWidget
         ui->colorValueSpin->setMaximum (50);
         ui->colorValueSpin->setValue (config.getDarkBgColorValue());
     }
-    connect (ui->colorValueSpin, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),
-             this, &PrefDialog::prefColValue);
+    connect (ui->colorValueSpin, QOverload<int>::of(&QSpinBox::valueChanged), this, &PrefDialog::prefColValue);
 
     ui->thickCursorBox->setChecked (config.getThickCursor());
 
@@ -228,15 +236,13 @@ PrefDialog::PrefDialog (const QHash<QString, QString> &defaultShortcuts, QWidget
     connect (ui->skipNonTextBox, &QCheckBox::stateChanged, this, &PrefDialog::prefSkipNontext);
 
     ui->spinBox->setValue (config.getMaxSHSize());
-    connect (ui->spinBox, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),
-             this, &PrefDialog::prefMaxSHSize);
+    connect (ui->spinBox, QOverload<int>::of(&QSpinBox::valueChanged), this, &PrefDialog::prefMaxSHSize);
 
     ui->inertiaBox->setChecked (config.getInertialScrolling());
     connect (ui->inertiaBox, &QCheckBox::stateChanged, this, &PrefDialog::prefInertialScrolling);
 
     ui->textTabSpin->setValue (textTabSize_);
-    connect (ui->textTabSpin, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),
-             this, &PrefDialog::prefTextTabSize);
+    connect (ui->textTabSpin, QOverload<int>::of(&QSpinBox::valueChanged), this, &PrefDialog::prefTextTabSize);
 
     /*************
      *** Files ***
@@ -251,8 +257,7 @@ PrefDialog::PrefDialog (const QHash<QString, QString> &defaultShortcuts, QWidget
 
     ui->recentSpin->setValue (config.getRecentFilesNumber());
     ui->recentSpin->setSuffix(" " + (ui->recentSpin->value() > 1 ? tr ("files") : tr ("file")));
-    connect (ui->recentSpin, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),
-             this, &PrefDialog::prefRecentFilesNumber);
+    connect (ui->recentSpin, QOverload<int>::of(&QSpinBox::valueChanged), this, &PrefDialog::prefRecentFilesNumber);
 
     ui->lastFilesBox->setChecked (config.getSaveLastFilesList());
     connect (ui->lastFilesBox, &QCheckBox::stateChanged, this, &PrefDialog::prefSaveLastFilesList);
@@ -271,121 +276,33 @@ PrefDialog::PrefDialog (const QHash<QString, QString> &defaultShortcuts, QWidget
      *** Shortcuts ***
      *****************/
 
-    defaultShortcuts_ = defaultShortcuts;
-    FPwin *win = static_cast<FPwin *>(parent_);
-
-    if (OBJECT_NAMES.isEmpty())
+    if (FPwin *win = static_cast<FPwin *>(parent_))
     {
-        OBJECT_NAMES.insert (win->ui->actionNew->text().remove ("&"), "actionNew");
-        OBJECT_NAMES.insert (win->ui->actionOpen->text().remove ("&"), "actionOpen");
-        OBJECT_NAMES.insert (win->ui->actionSave->text().remove ("&"), "actionSave");
-        OBJECT_NAMES.insert (win->ui->actionReload->text().remove ("&"), "actionReload");
-        OBJECT_NAMES.insert (win->ui->actionFind->text().remove ("&"), "actionFind");
-        OBJECT_NAMES.insert (win->ui->actionReplace->text().remove ("&"), "actionReplace");
-        OBJECT_NAMES.insert (win->ui->actionSaveAs->text().remove ("&"), "actionSaveAs");
-        OBJECT_NAMES.insert (win->ui->actionPrint->text().remove ("&"), "actionPrint");
-        OBJECT_NAMES.insert (win->ui->actionDoc->text().remove ("&"), "actionDoc");
-        OBJECT_NAMES.insert (win->ui->actionClose->text().remove ("&"), "actionClose");
-        OBJECT_NAMES.insert (win->ui->actionQuit->text().remove ("&"), "actionQuit");
-        OBJECT_NAMES.insert (win->ui->actionLineNumbers->text().remove ("&"), "actionLineNumbers");
-        OBJECT_NAMES.insert (win->ui->actionWrap->text().remove ("&"), "actionWrap");
-        OBJECT_NAMES.insert (win->ui->actionIndent->text().remove ("&"), "actionIndent");
-        OBJECT_NAMES.insert (win->ui->actionSyntax->text().remove ("&"), "actionSyntax");
-        OBJECT_NAMES.insert (win->ui->actionPreferences->text().remove ("&"), "actionPreferences");
-        OBJECT_NAMES.insert (win->ui->actionHelp->text().remove ("&"), "actionHelp");
-        OBJECT_NAMES.insert (win->ui->actionJump->text().remove ("&"), "actionJump");
-        OBJECT_NAMES.insert (win->ui->actionEdit->text().remove ("&"), "actionEdit");
-        OBJECT_NAMES.insert (win->ui->actionDetachTab->text().remove ("&"), "actionDetachTab");
-        OBJECT_NAMES.insert (win->ui->actionRun->text().remove ("&"), "actionRun");
-        OBJECT_NAMES.insert (win->ui->actionSession->text().remove ("&"), "actionSession");
-        OBJECT_NAMES.insert (win->ui->actionSidePane->text().remove ("&"), "actionSidePane");
-
-        OBJECT_NAMES.insert (win->ui->actionUpperCase->text().remove ("&"), "actionUpperCase");
-        OBJECT_NAMES.insert (win->ui->actionLowerCase->text().remove ("&"), "actionLowerCase");
-
-        OBJECT_NAMES.insert (win->ui->actionUndo->text().remove ("&"), "actionUndo");
-        OBJECT_NAMES.insert (win->ui->actionRedo->text().remove ("&"), "actionRedo");
-        OBJECT_NAMES.insert (win->ui->actionDate->text().remove ("&"), "actionDate");
-
-        OBJECT_NAMES.insert (win->ui->actionRightTab->text().remove ("&"), "actionRightTab");
-        OBJECT_NAMES.insert (win->ui->actionLeftTab->text().remove ("&"), "actionLeftTab");
-        OBJECT_NAMES.insert (win->ui->actionFirstTab->text().remove ("&"), "actionFirstTab");
-        OBJECT_NAMES.insert (win->ui->actionLastTab->text().remove ("&"), "actionLastTab");
-        OBJECT_NAMES.insert (win->ui->actionLastActiveTab->text().remove ("&"), "actionLastActiveTab");
+        if (DEFAULT_SHORTCUTS.isEmpty())
+        { // NOTE: Shortcut strings hould be in the PortableText format.
+            const auto defaultShortcuts = win->defaultShortcuts();
+            QHash<QAction*, QKeySequence>::const_iterator iter = defaultShortcuts.constBegin();
+            while (iter != defaultShortcuts.constEnd())
+            {
+                const QString name = iter.key()->objectName();
+                DEFAULT_SHORTCUTS.insert (name, iter.value().toString());
+                OBJECT_NAMES.insert (iter.key()->text().remove ("&"), name);
+                ++ iter;
+            }
+        }
     }
 
     QHash<QString, QString> ca = config.customShortcutActions();
+
     QList<QString> keys = ca.keys();
-
-    shortcuts_.insert (win->ui->actionNew->text().remove ("&"),
-                       keys.contains ("actionNew") ? ca.value ("actionNew") : defaultShortcuts_.value ("actionNew"));
-    shortcuts_.insert (win->ui->actionOpen->text().remove ("&"),
-                       keys.contains ("actionOpen") ? ca.value ("actionOpen") : defaultShortcuts_.value ("actionOpen"));
-    shortcuts_.insert (win->ui->actionSave->text().remove ("&"),
-                       keys.contains ("actionSave") ? ca.value ("actionSave") : defaultShortcuts_.value ("actionSave"));
-    shortcuts_.insert (win->ui->actionReload->text().remove ("&"),
-                       keys.contains ("actionReload") ? ca.value ("actionReload") : defaultShortcuts_.value ("actionReload"));
-    shortcuts_.insert (win->ui->actionFind->text().remove ("&"),
-                       keys.contains ("actionFind") ? ca.value ("actionFind") : defaultShortcuts_.value ("actionFind"));
-    shortcuts_.insert (win->ui->actionReplace->text().remove ("&"),
-                       keys.contains ("actionReplace") ? ca.value ("actionReplace") : defaultShortcuts_.value ("actionReplace"));
-    shortcuts_.insert (win->ui->actionSaveAs->text().remove ("&"),
-                       keys.contains ("actionSaveAs") ? ca.value ("actionSaveAs") : defaultShortcuts_.value ("actionSaveAs"));
-    shortcuts_.insert (win->ui->actionPrint->text().remove ("&"),
-                       keys.contains ("actionPrint") ? ca.value ("actionPrint") :defaultShortcuts_.value ("actionPrint"));
-    shortcuts_.insert (win->ui->actionDoc->text().remove ("&"),
-                       keys.contains ("actionDoc") ? ca.value ("actionDoc") : defaultShortcuts_.value ("actionDoc"));
-    shortcuts_.insert (win->ui->actionClose->text().remove ("&"),
-                       keys.contains ("actionClose") ? ca.value ("actionClose") : defaultShortcuts_.value ("actionClose"));
-    shortcuts_.insert (win->ui->actionQuit->text().remove ("&"),
-                       keys.contains ("actionQuit") ? ca.value ("actionQuit") : defaultShortcuts_.value ("actionQuit"));
-    shortcuts_.insert (win->ui->actionLineNumbers->text().remove ("&"),
-                       keys.contains ("actionLineNumbers") ? ca.value ("actionLineNumbers") : defaultShortcuts_.value ("actionLineNumbers"));
-    shortcuts_.insert (win->ui->actionWrap->text().remove ("&"),
-                       keys.contains ("actionWrap") ? ca.value ("actionWrap") : defaultShortcuts_.value ("actionWrap"));
-    shortcuts_.insert (win->ui->actionIndent->text().remove ("&"),
-                       keys.contains ("actionIndent") ? ca.value ("actionIndent") : defaultShortcuts_.value ("actionIndent"));
-    shortcuts_.insert (win->ui->actionSyntax->text().remove ("&"),
-                       keys.contains ("actionSyntax") ? ca.value ("actionSyntax") : defaultShortcuts_.value ("actionSyntax"));
-    shortcuts_.insert (win->ui->actionPreferences->text().remove ("&"),
-                       keys.contains ("actionPreferences") ? ca.value ("actionPreferences") : defaultShortcuts_.value ("actionPreferences"));
-    shortcuts_.insert (win->ui->actionHelp->text().remove ("&"),
-                       keys.contains ("actionHelp") ? ca.value ("actionHelp") : defaultShortcuts_.value ("actionHelp"));
-    shortcuts_.insert (win->ui->actionJump->text().remove ("&"),
-                       keys.contains ("actionJump") ? ca.value ("actionJump") : defaultShortcuts_.value ("actionJump"));
-    shortcuts_.insert (win->ui->actionEdit->text().remove ("&"),
-                       keys.contains ("actionEdit") ? ca.value ("actionEdit") : defaultShortcuts_.value ("actionEdit"));
-    shortcuts_.insert (win->ui->actionDetachTab->text().remove ("&"),
-                       keys.contains ("actionDetachTab") ? ca.value ("actionDetachTab") : defaultShortcuts_.value ("actionDetachTab"));
-    shortcuts_.insert (win->ui->actionRun->text().remove ("&"),
-                       keys.contains ("actionRun") ? ca.value ("actionRun") : defaultShortcuts_.value ("actionRun"));
-    shortcuts_.insert (win->ui->actionSession->text().remove ("&"),
-                       keys.contains ("actionSession") ? ca.value ("actionSession") : defaultShortcuts_.value ("actionSession"));
-    shortcuts_.insert (win->ui->actionSidePane->text().remove ("&"),
-                       keys.contains ("actionSidePane") ? ca.value ("actionSidePane") : defaultShortcuts_.value ("actionSidePane"));
-
-    shortcuts_.insert (win->ui->actionUpperCase->text().remove ("&"),
-                       keys.contains ("actionUpperCase") ? ca.value ("actionUpperCase") : defaultShortcuts_.value ("actionUpperCase"));
-    shortcuts_.insert (win->ui->actionLowerCase->text().remove ("&"),
-                       keys.contains ("actionLowerCase") ? ca.value ("actionLowerCase") : defaultShortcuts_.value ("actionLowerCase"));
-
-    shortcuts_.insert (win->ui->actionUndo->text().remove ("&"),
-                       keys.contains ("actionUndo") ? ca.value ("actionUndo") : defaultShortcuts_.value ("actionUndo"));
-    shortcuts_.insert (win->ui->actionRedo->text().remove ("&"),
-                       keys.contains ("actionRedo") ? ca.value ("actionRedo") : defaultShortcuts_.value ("actionRedo"));
-    shortcuts_.insert (win->ui->actionDate->text().remove ("&"),
-                       keys.contains ("actionDate") ? ca.value ("actionDate") : defaultShortcuts_.value ("actionDate"));
-
-    shortcuts_.insert (win->ui->actionRightTab->text().remove ("&"),
-                       keys.contains ("actionRightTab") ? ca.value ("actionRightTab") : defaultShortcuts_.value ("actionRightTab"));
-    shortcuts_.insert (win->ui->actionLeftTab->text().remove ("&"),
-                       keys.contains ("actionLeftTab") ? ca.value ("actionLeftTab") : defaultShortcuts_.value ("actionLeftTab"));
-    shortcuts_.insert (win->ui->actionFirstTab->text().remove ("&"),
-                       keys.contains ("actionFirstTab") ? ca.value ("actionFirstTab") : defaultShortcuts_.value ("actionFirstTab"));
-    shortcuts_.insert (win->ui->actionLastTab->text().remove ("&"),
-                       keys.contains ("actionLastTab") ? ca.value ("actionLastTab") : defaultShortcuts_.value ("actionLastTab"));
-    shortcuts_.insert (win->ui->actionLastActiveTab->text().remove ("&"),
-                       keys.contains ("actionLastActiveTab") ? ca.value ("actionLastActiveTab") : defaultShortcuts_.value ("actionLastActiveTab"));
+    QHash<QString, QString>::const_iterator iter = OBJECT_NAMES.constBegin();
+    while (iter != OBJECT_NAMES.constEnd())
+    {
+        shortcuts_.insert (iter.key(),
+                           keys.contains (iter.value()) ? ca.value (iter.value())
+                                                        : DEFAULT_SHORTCUTS.value (iter.value()));
+        ++ iter;
+    }
 
     QList<QString> val = shortcuts_.values();
     for (int i = 0; i < val.size(); ++i)
@@ -406,14 +323,16 @@ PrefDialog::PrefDialog (const QHash<QString, QString> &defaultShortcuts, QWidget
         QTableWidgetItem *item = new QTableWidgetItem (it.key());
         item->setFlags (item->flags() & ~Qt::ItemIsEditable & ~Qt::ItemIsSelectable);
         ui->tableWidget->setItem (index, 0, item);
-        ui->tableWidget->setItem (index, 1, new QTableWidgetItem (it.value()));
+        /* shortcut texts should added in the NativeText format */
+        ui->tableWidget->setItem (index, 1, new QTableWidgetItem (QKeySequence (it.value(), QKeySequence::PortableText)
+                                                                  .toString (QKeySequence::NativeText)));
         ++ it;
         ++ index;
     }
     ui->tableWidget->setSortingEnabled (true);
     ui->tableWidget->setCurrentCell (0, 1);
     connect (ui->tableWidget, &QTableWidget::itemChanged, this, &PrefDialog::onShortcutChange);
-    connect (ui->defaultButton, &QAbstractButton::clicked, this, &PrefDialog::defaultSortcuts);
+    connect (ui->defaultButton, &QAbstractButton::clicked, this, &PrefDialog::restoreDefaultShortcuts);
     ui->defaultButton->setDisabled (ca.isEmpty());
 
     /*************
@@ -425,11 +344,16 @@ PrefDialog::PrefDialog (const QHash<QString, QString> &defaultShortcuts, QWidget
     connect (this, &QDialog::rejected, this, &PrefDialog::onClosing);
 
     /* set tooltip as "whatsthis" */
-    QList<QWidget*> widgets = findChildren<QWidget*>();
-    for (int i = 0; i < widgets.count(); ++i)
+    const auto widgets = findChildren<QWidget*>();
+    for (QWidget *w : widgets)
     {
-        QWidget *w = widgets.at (i);
-        w->setWhatsThis (w->toolTip().replace ('\n', ' ').replace ("  ", "\n\n"));
+        QString tip = w->toolTip();
+        if (!tip.isEmpty())
+        {
+            w->setWhatsThis (tip.replace ('\n', ' ').replace ("  ", "\n\n"));
+            /* for the tooltip mess in Qt 5.12 */
+            w->setToolTip ("<p style='white-space:pre'>" + w->toolTip() + "</p>");
+        }
     }
 
     resize (sizeHint() + QSize (style()->pixelMetric(QStyle::PM_ScrollBarExtent), 0));
@@ -505,7 +429,7 @@ void PrefDialog::showPrompt (const QString& str, bool temporary)
              || (!darkBg_ && lightColValue_ != config.getLightBgColorValue())
              || showWhiteSpace_ != config.getShowWhiteSpace()
              || showEndings_ != config.getShowEndings()
-             || textTabSize_ != config.getTextTab().size()
+             || textTabSize_ != config.getTextTabSize()
              || (vLineDistance_ * config.getVLineDistance() < 0
                  || (vLineDistance_ > 0 && vLineDistance_ != config.getVLineDistance())))
     {
@@ -957,8 +881,7 @@ void PrefDialog::prefEndings (int checked)
 void PrefDialog::prefDarkColScheme (int checked)
 {
     Config& config = static_cast<FPsingleton*>(qApp)->getConfig();
-    disconnect (ui->colorValueSpin, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),
-                this, &PrefDialog::prefColValue);
+    disconnect (ui->colorValueSpin, QOverload<int>::of(&QSpinBox::valueChanged), this, &PrefDialog::prefColValue);
     if (checked == Qt::Checked)
     {
         config.setDarkColScheme (true);
@@ -973,8 +896,7 @@ void PrefDialog::prefDarkColScheme (int checked)
         ui->colorValueSpin->setMaximum (255);
         ui->colorValueSpin->setValue (config.getLightBgColorValue());
     }
-    connect (ui->colorValueSpin, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),
-             this, &PrefDialog::prefColValue);
+    connect (ui->colorValueSpin, QOverload<int>::of(&QSpinBox::valueChanged), this, &PrefDialog::prefColValue);
 
     showPrompt();
 }
@@ -1266,17 +1188,41 @@ void PrefDialog::prefSplitterPos (int checked)
         config.setRemSplitterPos (false);
 }
 /*************************/
+// NOTE: Custom shortcuts will be saved in the PortableText format.
 void PrefDialog::onShortcutChange (QTableWidgetItem *item)
 {
     Config config = static_cast<FPsingleton*>(qApp)->getConfig();
+
     QString txt = item->text();
+    bool invalid (txt.isEmpty());
+    if (!invalid)
+    {
+        /* the QKeySequenceEdit text is in the NativeText format but it should be
+           converted into the PortableText format, both for saving and checking for "+" */
+        QKeySequence keySeq (txt);
+        txt = keySeq.toString();
+        if (!txt.contains ("+"))
+        {
+            invalid = true;
+            /* make and exception for Fx keys */
+            for (int i = Qt::Key_F1; i <= Qt::Key_F35; ++i)
+            {
+                if (QKeySequence::ExactMatch == keySeq.matches (QKeySequence (i)))
+                {
+                    invalid = false;
+                    break;
+                }
+            }
+        }
+    }
+
     QString desc = ui->tableWidget->item (ui->tableWidget->currentRow(), 0)->text();
-    if (txt.isEmpty() || !txt.contains ("+")
+    if (invalid
         || (config.reservedShortcuts().contains (txt)
             /* unless its (hard-coded) default shortcut is typed */
-            && defaultShortcuts_.value (OBJECT_NAMES.value (desc)) != txt))
+            && DEFAULT_SHORTCUTS.value (OBJECT_NAMES.value (desc)) != txt))
     {
-        if (txt.isEmpty() || !txt.contains ("+"))
+        if (invalid)
             showPrompt (tr ("The typed shortcut was not valid."), true);
         else
             showPrompt (tr ("The typed shortcut was reserved."), true);
@@ -1299,7 +1245,7 @@ void PrefDialog::onShortcutChange (QTableWidgetItem *item)
         QHash<QString, QString>::const_iterator it = shortcuts_.constBegin();
         while (it != shortcuts_.constEnd())
         {
-            if (defaultShortcuts_.value (OBJECT_NAMES.value (it.key())) != it.value())
+            if (DEFAULT_SHORTCUTS.value (OBJECT_NAMES.value (it.key())) != it.value())
             {
                 ui->defaultButton->setEnabled (true);
                 return;
@@ -1310,7 +1256,7 @@ void PrefDialog::onShortcutChange (QTableWidgetItem *item)
     }
 }
 /*************************/
-void PrefDialog::defaultSortcuts()
+void PrefDialog::restoreDefaultShortcuts()
 {
     if (newShortcuts_.isEmpty()
         && static_cast<FPsingleton*>(qApp)->getConfig().customShortcutActions().isEmpty())
@@ -1323,14 +1269,14 @@ void PrefDialog::defaultSortcuts()
                   ? 0
                   : ui->tableWidget->currentRow();
     ui->tableWidget->setSortingEnabled (false);
-    newShortcuts_ = defaultShortcuts_;
+    newShortcuts_ = DEFAULT_SHORTCUTS;
     int index = 0;
     QMutableHashIterator<QString, QString> it (shortcuts_);
     while (it.hasNext())
     {
         it.next();
         ui->tableWidget->item (index, 0)->setText (it.key());
-        QString s = defaultShortcuts_.value (OBJECT_NAMES.value (it.key()));
+        QString s = DEFAULT_SHORTCUTS.value (OBJECT_NAMES.value (it.key()));
         ui->tableWidget->item (index, 1)->setText (s);
         it.setValue (s);
         ++ index;
@@ -1354,7 +1300,7 @@ void PrefDialog::prefShortcuts()
     QHash<QString, QString>::const_iterator it = newShortcuts_.constBegin();
     while (it != newShortcuts_.constEnd())
     {
-        if (defaultShortcuts_.value (it.key()) == it.value())
+        if (DEFAULT_SHORTCUTS.value (it.key()) == it.value())
             config.removeShortcut (it.key());
         else
             config.setActionShortcut (it.key(), it.value());
@@ -1421,7 +1367,7 @@ void PrefDialog::prefApplyAutoSave()
 }
 /*************************/
 void PrefDialog::prefTextTabSize (int value)
-{
+{ // textTabSize_ is updated but the config isn't
     if (value >= 2 && value <= 10)
     {
         textTabSize_ = value;
@@ -1430,14 +1376,9 @@ void PrefDialog::prefTextTabSize (int value)
 }
 /*************************/
 void PrefDialog::prefTextTab()
-{ // textTabSize_ is updated but the config isn't
+{
     Config& config = static_cast<FPsingleton*>(qApp)->getConfig();
-    if (textTabSize_ == config.getTextTab().size())
-        return;
-    QString textTab;
-    for (int i = 0; i < textTabSize_; ++i)
-        textTab += " ";
-    config.setTextTab (textTab);
+    config.setTextTabSize (textTabSize_);
 }
 /*************************/
 void PrefDialog::prefCloseWithLastTab (int checked)
