@@ -1,5 +1,5 @@
 /*
- * Copyright (C) Pedram Pourang (aka Tsu Jan) 2014 <tsujan2000@gmail.com>
+ * Copyright (C) Pedram Pourang (aka Tsu Jan) 2014-2019 <tsujan2000@gmail.com>
  *
  * FeatherPad is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -68,7 +68,7 @@ void FPwin::setProgLang (TextEdit *textEdit)
             progLan = "cpp";
         else if (fname.endsWith (".c"))
             progLan = "c";
-        else if (fname.endsWith (".sh") || fname.endsWith (".bashrc")
+        else if (fname.endsWith (".sh") || fname.endsWith (".bashrc") || fname.endsWith (".rules")
                  || baseName == ".bash_profile" || baseName == ".bash_functions"
                  || baseName == ".xprofile" || baseName == ".profile"
                  || baseName == ".bash_aliases" || baseName == ".mkshrc"
@@ -102,6 +102,7 @@ void FPwin::setProgLang (TextEdit *textEdit)
         else if (fname.endsWith (".kvconfig")
                  || fname.endsWith (".service") || fname.endsWith (".mount") || fname.endsWith (".timer") // systemd related
                  || baseName == "sources.list" || baseName == "sources.list.save"
+                 || baseName == "mimeinfo.cache" || baseName == "mimeapps.list" || baseName.endsWith ("-mimeapps.list")
                  || fname.endsWith (".pls", Qt::CaseInsensitive))
              progLan = "config";
         else if (fname.endsWith (".js"))
@@ -124,12 +125,14 @@ void FPwin::setProgLang (TextEdit *textEdit)
             progLan = "html";
         else if (fname.endsWith (".markdown") || fname.endsWith (".md") || fname.endsWith (".mkd"))
             progLan = "markdown";
+        else if (fname.endsWith (".rst"))
+            progLan = "reST";
         else if (baseName.startsWith ("makefile.", Qt::CaseInsensitive) && !baseName.endsWith (".txt", Qt::CaseInsensitive))
             progLan = "makefile";
         else if (baseName.compare ("CMakeLists.txt", Qt::CaseInsensitive) == 0)
             progLan = "cmake";
     }
-    else if (baseName == "PKGBUILD")
+    else if (baseName == "PKGBUILD" || baseName == "fstab")
         progLan = "sh";
     /* makefile is an exception */
     else if (baseName.compare ("makefile", Qt::CaseInsensitive) == 0)
@@ -210,6 +213,8 @@ void FPwin::setProgLang (TextEdit *textEdit)
                 progLan = "diff";
             else if (mime == "text/html" || mime == "application/xhtml+xml")
                 progLan = "html";
+            else if (mime == "text/markdown")
+                progLan = "markdown";
             else if (mime == "audio/x-mpegurl")
                 progLan = "m3u";
             else if (fname.endsWith (".conf") || fname.endsWith (".ini"))
@@ -316,7 +321,7 @@ void FPwin::formatOnTextChange (int /*position*/, int charsRemoved, int charsAdd
     {
         /* wait until the document's layout manager is notified about the change;
            otherwise, the end cursor might be out of range in formatTextRect() */
-        QTimer::singleShot (0, this, SLOT (formatOnResizing()));
+        QTimer::singleShot (0, this, &FPwin::formatOnResizing);
     }
 }
 /*************************/
